@@ -28,7 +28,7 @@ static void set_k8_name(struct k8_rev *r, struct cpudata *c)
 
 	s[0] = 0;
 	cont = 0;
-	for (i=0; (r != NULL) && (i<ARRAY_SIZE(k8_names)); i++) {
+	for (i = 0; (r != NULL) && (i < ARRAY_SIZE(k8_names)); i++) {
 		p = NULL;
 		id = 1<<i;
 		if (r->nameid & id)
@@ -54,7 +54,7 @@ static void set_k8_revinfo(int id, struct cpudata *c)
 
 	c->connector = 0;
 	r = NULL;
-	for (i=0; i<ARRAY_SIZE(k8_revisions); i++) {
+	for (i = 0; i < ARRAY_SIZE(k8_revisions); i++) {
 		if (k8_revisions[i].eax == id) {
 			r = &k8_revisions[i];
 			break;
@@ -73,13 +73,9 @@ static void set_fam10h_name(struct fam10h_rev *r, struct cpudata *c)
 		return;
 	}
 
-	if (r->nameid & _PHENOM)
-		snprintf(c->name, CPU_NAME_LEN,
-			 "Quad-Core Opteron/Phenom (%s)",
-			 r->rev);
-	else if (r->nameid)
-		snprintf(c->name, CPU_NAME_LEN,
-			 "Quad-Core Opteron (%s)", r->rev);
+	snprintf(c->name, CPU_NAME_LEN,
+		 "Phenom/Athlon/Sempron/Turion (II)/Opteron (%s)",
+		 r->rev);
 }
 
 static void set_connector(struct cpudata *c)
@@ -87,21 +83,59 @@ static void set_connector(struct cpudata *c)
 	unsigned int eax, ebx, ecx, edx;
 	int pkg_id;
 
+	if (family(c) == 0x11) {
+		c->connector = CONN_SOCKET_S1G2;
+		return;
+	}
+
 	cpuid(c->number, 0x80000001, &eax, &ebx, &ecx, &edx);
 	pkg_id = (ebx >> 28) & 0xf;
 
-	switch (pkg_id) {
-	case 0:
-		c->connector = CONN_SOCKET_F_R2;
-		break;
-	case 1:
-		c->connector = CONN_SOCKET_AM2_R2;
-		break;
-	case 2:
-		c->connector = CONN_SOCKET_S1G2;
-		break;
-	default:
-		c->connector = 0;
+	if ((family(c) == 0x10) || (family(c) == 0x11) || (family(c) == 0x15)) {
+		switch (pkg_id) {
+		case 0:
+			c->connector = CONN_SOCKET_F_R2;
+			break;
+		case 1:
+			c->connector = CONN_SOCKET_AM3;
+			break;
+		case 2:
+			c->connector = CONN_SOCKET_S1G3;
+			break;
+		case 3:
+			c->connector = CONN_SOCKET_G34;
+			break;
+		case 4:
+			c->connector = CONN_SOCKET_ASB2;
+			break;
+		case 5:
+			c->connector = CONN_SOCKET_C32;
+			break;
+		default:
+			c->connector = 0;
+		}
+	} else if(family(c) == 0x12) {
+		switch (pkg_id) {
+		case 0:
+			c->connector = CONN_SOCKET_FP1;
+			break;
+		case 1:
+			c->connector = CONN_SOCKET_FS1;
+			break;
+		case 2:
+			c->connector = CONN_SOCKET_FM1;
+			break;
+		default:
+			c->connector = 0;
+		}
+	} else if (family(c) == 0x14) {
+		switch (pkg_id) {
+		case 0:
+			c->connector = CONN_SOCKET_FT1;
+			break;
+		default:
+			c->connector = 0;
+		}
 	}
 }
 
@@ -110,7 +144,7 @@ static void set_fam10h_revinfo(int id, struct cpudata *c)
 	int i;
 	struct fam10h_rev *r = NULL;
 
-	for (i=0; i<ARRAY_SIZE(fam10h_revisions); i++) {
+	for (i = 0; i < ARRAY_SIZE(fam10h_revisions); i++) {
 		if (fam10h_revisions[i].eax == id) {
 			r = &fam10h_revisions[i];
 			break;
@@ -125,7 +159,7 @@ static void set_fam11h_revinfo(int id, struct cpudata *c)
 	const char *p;
 
 	p = get_fam11h_revision_name(id);
-	if(p)
+	if (p)
 		snprintf(c->name, CPU_NAME_LEN,
 			 "AMD Turion X2 Ultra Dual-Core (%s)", p);
 	else
@@ -134,6 +168,49 @@ static void set_fam11h_revinfo(int id, struct cpudata *c)
 	set_connector(c);
 }
 
+static void set_fam12h_revinfo(int id, struct cpudata *c)
+{
+	const char *p;
+
+	p = get_fam12h_revision_name(id);
+	if(p)
+		snprintf(c->name, CPU_NAME_LEN,
+			 "AMD A/E2-Series Processor (%s)", p);
+	else
+		snprintf(c->name, CPU_NAME_LEN, "Unknown CPU");
+
+	set_connector(c);
+}
+
+static void set_fam14h_revinfo(int id, struct cpudata *c)
+{
+	const char *p;
+
+	p = get_fam14h_revision_name(id);
+	if (p)
+		snprintf(c->name, CPU_NAME_LEN,
+			 "AMD C/E/G-Series Processor (%s)", p);
+	else
+		snprintf(c->name, CPU_NAME_LEN, "Unknown CPU");
+
+
+	set_connector(c);
+}
+
+static void set_fam15h_revinfo(int id, struct cpudata *c)
+{
+	const char *p;
+
+	p = get_fam15h_revision_name(id);
+	if (p)
+		snprintf(c->name, CPU_NAME_LEN,
+			 "AMD FX Series Processor (%s)", p);
+	else
+		snprintf(c->name, CPU_NAME_LEN, "Unknown CPU");
+	set_connector(c);
+}
+
+
 static void do_assoc(unsigned long assoc)
 {
 	switch (assoc & 0xff) {
@@ -285,15 +362,19 @@ void decode_AMD_cacheinfo(struct cpudata *cpu)
 		printf("\n\t");
 		printf("lines per tag=%u\t", (ecx >> 8) & 0x0f);
 		printf("line size=%u bytes.\n", ecx & 0xff);
-		if (family(cpu) == 0x10) {
-			/* family 0x10 has shared L3  cache */
+		if ((family(cpu) == 0x10) || (family(cpu) == 0x15)) {
 			printf("L3 (shared) cache:\n\t");
-			printf("Size: %uKb\t",
-			       (edx >> 18) * 512);
-			do_l2assoc((edx >> 12) & 0x0f);
-			printf("\n\t");
-			printf("lines per tag=%u\t", (edx >> 8) & 0x0f);
-			printf("line size=%u bytes.\n", edx & 0xff);
+			if (!(edx >> 18))
+				printf("none/disabled\n");
+			else {
+				/* shared L3  cache */
+				printf("Size: %uKb\t",
+				       (edx >> 18) * 512);
+				do_l2assoc((edx >> 12) & 0x0f);
+				printf("\n\t");
+				printf("lines per tag=%u\t", (edx >> 8) & 0x0f);
+				printf("line size=%u bytes.\n", edx & 0xff);
+			}
 		}
 	}
 
@@ -353,11 +434,11 @@ void Identify_AMD(struct cpudata *cpu)
 
 	amd_nameptr = cpu->name;
 
-	if (cpu->maxi < 1)
+	if (cpu->cpuid_level < 1)
 		return;
 
+	cpuid(cpu->number, 0x00000001, &eax, &ebx, &ecx, &edx);
 	if (cpu->family == 0xf) {
-		cpuid(cpu->number, 0x00000001, &eax, &ebx, &ecx, &edx);
 		cpu->emodel = (eax >> 16) & 0xf;
 		cpu->efamily= (eax >> 20) & 0xff;
 	} else {
@@ -365,14 +446,32 @@ void Identify_AMD(struct cpudata *cpu)
 		cpu->efamily = 0;
 	}
 
-	if (family(cpu) == 0xf) {
-		set_k8_revinfo(eax, cpu);
-		return;
-	} else if (family(cpu) == 0x10) {
-		set_fam10h_revinfo(eax, cpu);
-		return;
-	} else if (family(cpu) == 0x11) {
-		set_fam11h_revinfo(eax, cpu);
+	if (family(cpu) >= 0xf) {
+		switch family(cpu) {
+		case 0xf:
+			set_k8_revinfo(eax, cpu);
+			break;
+		case 0x10:
+			set_fam10h_revinfo(eax, cpu);
+			break;
+		case 0x11:
+			set_fam11h_revinfo(eax, cpu);
+			break;
+		case 0x12:
+			set_fam12h_revinfo(eax, cpu);
+			break;
+		case 0x14:
+			set_fam14h_revinfo(eax, cpu);
+			break;
+		case 0x15:
+			set_fam15h_revinfo(eax, cpu);
+			break;
+		default:
+			printf("Unknown CPU family: 0x%x\n",
+			       family(cpu));
+			break;
+		}
+
 		return;
 	}
 
@@ -699,6 +798,19 @@ out_660:
 	}
 }
 
+static void show_patch_level(struct cpudata *cpu)
+{
+	unsigned long long val = 0;
+	if (!user_is_root)
+		return;
+
+	if (read_msr(cpu->number, 0x8b, &val) == 1) {
+		if (val>0)
+			printf("Microcode patch level: 0x%llx\n", val);
+		printf("\n");
+	}
+}
+
 void display_AMD_info(struct cpudata *cpu)
 {
 	unsigned int eax, ebx, ecx, edx;
@@ -710,8 +822,11 @@ void display_AMD_info(struct cpudata *cpu)
 			dump_athlon_MSR(cpu);
 	}
 
-	if (show_bluesmoke)
-		decode_athlon_bluesmoke(cpu->number);
+	if (show_machine_check)
+		decode_athlon_machine_check(cpu->number);
+
+	if (show_microcode && family(cpu) >= 0xf)
+		show_patch_level(cpu);
 
 	if (show_pm)
 		decode_powernow(cpu);
@@ -719,7 +834,7 @@ void display_AMD_info(struct cpudata *cpu)
 	if (show_bugs)
 		show_amd_bugs(cpu);
 
-	if (cpu->maxi >= 0x05) {
+	if (cpu->cpuid_level >= 0x05) {
 		cpuid (cpu->number, 0x05, &eax, &ebx, &ecx, NULL);
 		printf("Monitor/Mwait: min/max line size %d/%d%s%s\n",
 		       (int) (eax & 0xffff), (int) (ebx & 0xffff),
@@ -739,6 +854,18 @@ void display_AMD_info(struct cpudata *cpu)
 			printf(", SVMLock");
 		if (edx & 8)
 			printf(", NRIPSave");
+		if (edx & 0x10)
+			printf(", TscRateMsr");
+		if (edx & 0x20)
+			printf(", VmcbClean");
+		if (edx & 0x40)
+			printf(", FlushByAsid");
+		if (edx & 0x80)
+			printf(", DecodeAssists");
+		if (edx & 0x400)
+			printf(", PauseFilter");
+		if (edx & 0x1000)
+			printf(", PauseFilterThreshold");
 		printf("\n");
 	}
 
