@@ -121,20 +121,25 @@ struct k8_rev k8_revisions[] = {
 	{0xc0f13, CONN_SOCKET_F, _ATHLON64_FX, "JH-F3"},
 };
 
-#define _OPTERON	0x0001
-#define _PHENOM		0x0002
-
 struct fam10h_rev {
 	int eax;
-	int nameid;
 	const char* rev;
 };
 
 struct fam10h_rev fam10h_revisions[] = {
-	{0x0100f2a, _OPTERON, "DR-BA"},
-	{0x0100f22, _OPTERON|_PHENOM, "DR-B2"},
-	{0x0100f23, _OPTERON|_PHENOM, "DR-B3"},
-	{0x0100f42, _OPTERON, "RB-C2"},
+	{0x0100f2a, "DR-BA"},
+	{0x0100f22, "DR-B2"},
+	{0x0100f23, "DR-B3"},
+	{0x0100f42, "RB-C2"},
+	{0x0100f43, "RB-C3"},
+	{0x0100f52, "BL-C2"},
+	{0x0100f53, "BL-C3"},
+	{0x0100f62, "DA-C2"},
+	{0x0100f63, "DA-C3"},
+	{0x0100f80, "HY-D0"},
+	{0x0100f81, "HY-D1"},
+	{0x0100f91, "HY-D1"},
+	{0x0100fa0, "PH-E0"},
 };
 
 struct id_string fam11h_revisions[] = {
@@ -142,4 +147,20 @@ struct id_string fam11h_revisions[] = {
 };
 get_name(fam11h_revision, int, fam11h_revisions);
 
+struct id_string fam12h_revisions[] = {
+	{0x0300f10, "LN1-B0"},
+};
+get_name(fam12h_revision, int, fam12h_revisions);
+
+struct id_string fam14h_revisions[] = {
+	{0x00500f10, "ON-B0"},
+};
+get_name(fam14h_revision, int, fam14h_revisions);
+
+struct id_string fam15h_revisions[] = {
+
+	{0x00600f12, "OR-B2"},
+};
+get_name(fam15h_revision, int, fam15h_revisions);
+
 #endif /* _amd_revision_h_ */
