@@ -1,6 +1,13 @@
-#include <stdio.h>
+/*
+ *  (C) 2001 Dave Jones.
+ *
+ *  Licensed under the terms of the GNU GPL License version 2.
+ */
 
-static __inline__ int flag_is_changeable_p(unsigned long flag)
+#include <stdio.h>
+#include "x86info.h"
+
+static int flag_is_changeable_p(unsigned long flag)
 {
 	unsigned long f1, f2;
 	__asm__ volatile("pushf\n\t"
