@@ -4,6 +4,6 @@
 #define	SMP_NO	0
 #define SMP_YES	1
 
-int issmp(int verbose);
+void display_mptable();
 
 #endif
